@@ -32,6 +32,7 @@
 #include <gdk/gdk.h>
 #include <errno.h>
 
+#include "plugin.h"
 #include <libanjuta/resources.h>
 #include <libanjuta/anjuta-utils.h>
 #include <libanjuta/anjuta-encodings.h>
@@ -60,6 +61,7 @@
 #include <libanjuta/interfaces/ianjuta-indicable.h>
 #include <libanjuta/interfaces/ianjuta-print.h>
 #include <libanjuta/interfaces/ianjuta-document.h>
+#include <libanjuta/interfaces/ianjuta-symbol-manager.h>
 
 #include "properties.h"
 #include "text_editor.h"
@@ -110,8 +112,7 @@ static GHashTable *supported_languages_by_lexer = NULL;
 
 static void text_editor_finalize (GObject *obj);
 static void text_editor_dispose (GObject *obj);
-static void text_editor_hilite_one (TextEditor * te, AnEditorID editor,
-									gboolean force);
+static void text_editor_hilite_one (TextEditor * te, AnEditorID editor);
 
 static GtkVBoxClass *parent_class;
 
@@ -126,6 +127,7 @@ text_editor_instance_init (TextEditor *te)
 	te->monitor = NULL;
 	te->preferences = NULL;
 	te->force_hilite = NULL;
+	te->force_pref = FALSE;
 	te->freeze_count = 0;
 	te->current_line = 0;
 	te->popup_menu = NULL;
@@ -258,7 +260,7 @@ text_editor_add_view (TextEditor *te)
 				G_CALLBACK (on_text_editor_scintilla_focus_in), te);
 	
 	initialize_markers (te, scintilla);
-	text_editor_hilite_one (te, editor_id, FALSE);
+	text_editor_hilite_one (te, editor_id);
 	text_editor_set_line_number_width (te);
 	
 	if (current_line)
@@ -509,6 +511,19 @@ text_editor_update_monitor (TextEditor *te, gboolean disable_it)
 	}
 }
 
+static void
+on_shell_value_changed  (TextEditor *te, const char *name)
+{
+	g_return_if_fail (name != NULL);
+	
+	if ((strcmp (name, TEXT_EDITOR_PROJECT_TYPE_LIST) == 0) ||
+	    (strcmp (name, TEXT_EDITOR_SYSTEM_TYPE_LIST) == 0))
+	{
+		/* Type names list has changed, so refresh highlight */
+		text_editor_hilite (te, te->force_pref);
+	}
+}
+
 GtkWidget *
 text_editor_new (AnjutaStatus *status, AnjutaPreferences *eo, AnjutaShell *shell, const gchar *uri, const gchar *name)
 {
@@ -561,6 +576,10 @@ text_editor_new (AnjutaStatus *status, AnjutaPreferences *eo, AnjutaShell *shell
 	zoom_factor = anjuta_preferences_get_int (te->preferences, TEXT_ZOOM_FACTOR);
 	/* DEBUG_PRINT ("%s", "Initializing zoom factor to: %d", zoom_factor); */
 	text_editor_set_zoom_factor (te, zoom_factor);
+
+	/* Get type name notification */
+	g_signal_connect_swapped (G_OBJECT (shell), "value-added", G_CALLBACK (on_shell_value_changed), te);
+	g_signal_connect_swapped (G_OBJECT (shell), "value-removed", G_CALLBACK (on_shell_value_changed), te);
 	
 #ifdef DEBUG
 	g_object_weak_ref (G_OBJECT (te), on_te_already_destroyed, te);
@@ -572,6 +591,10 @@ void
 text_editor_dispose (GObject *obj)
 {
 	TextEditor *te = TEXT_EDITOR (obj);
+
+	/* Disconnect signal */
+	g_signal_handlers_disconnect_by_func (te->shell, G_CALLBACK (on_shell_value_changed), te);
+	
 	if (te->monitor)
 	{
 		text_editor_update_monitor (te, TRUE);
@@ -668,47 +691,65 @@ text_editor_set_hilite_type (TextEditor * te, const gchar *file_extension)
 }
 
 static void
-text_editor_hilite_one (TextEditor * te, AnEditorID editor_id,
-						gboolean override_by_pref)
+text_editor_hilite_one (TextEditor * te, AnEditorID editor_id)
 {
-	/* If syntax highlighting is disabled ... */
-	if (override_by_pref &&
-		anjuta_preferences_get_bool (ANJUTA_PREFERENCES (te->preferences),
+	const gchar *name = NULL;
+	gchar *basename = NULL;
+	
+	/* syntax highlighting is disabled if te->force_pref && pref is disabled */
+	if (!te->force_pref ||
+		!anjuta_preferences_get_bool (ANJUTA_PREFERENCES (te->preferences),
 									DISABLE_SYNTAX_HILIGHTING))
 	{
-		aneditor_command (editor_id, ANE_SETHILITE, (glong) "plain.txt", 0);
+		if (te->force_hilite)
+		{
+			name = te->force_hilite;
+		}
+		else if (te->uri)
+		{
+			basename = g_path_get_basename (te->uri);
+			name = basename;
+		}
+		else if (te->filename)
+		{
+			name = te->filename;
+		}
 	}
-	else if (te->force_hilite)
+
+	if (name == NULL)
 	{
-		aneditor_command (editor_id, ANE_SETHILITE, (glong) te->force_hilite, 0);
-	}
-	else if (te->uri)
-	{
-		gchar *basename;
-		basename = g_path_get_basename (te->uri);
-		aneditor_command (editor_id, ANE_SETHILITE, (glong) basename, 0);
-		g_free (basename);
-	}
-	else if (te->filename)
-	{
-		aneditor_command (editor_id, ANE_SETHILITE, (glong) te->filename, 0);
+		/* No syntax higlight */
+		aneditor_command (editor_id, ANE_SETHILITE, (glong) "plain.txt", (glong) 0);
 	}
 	else
 	{
-		aneditor_command (editor_id, ANE_SETHILITE, (glong) "plain.txt", 0);
-	} 
+		const gchar *typedef_hl[2];
+		GValue sys_value = {0,};
+		GValue prj_value = {0,};
+
+		anjuta_shell_get_value (te->shell, TEXT_EDITOR_SYSTEM_TYPE_LIST, &sys_value, NULL);
+		typedef_hl[0] = G_VALUE_HOLDS_STRING(&sys_value) ? g_value_get_string (&sys_value) : NULL;
+
+		anjuta_shell_get_value (te->shell, TEXT_EDITOR_PROJECT_TYPE_LIST, &prj_value, NULL);
+		typedef_hl[1] = G_VALUE_HOLDS_STRING(&prj_value) ? g_value_get_string (&prj_value) : NULL;
+
+		aneditor_command (editor_id, ANE_SETHILITE, (glong) name, (glong) typedef_hl);
+		if (G_IS_VALUE (&sys_value)) g_value_unset (&sys_value);
+		if (G_IS_VALUE (&prj_value)) g_value_unset (&prj_value);
+	}
+	g_free (basename);
 }
 
 void
 text_editor_hilite (TextEditor * te, gboolean override_by_pref)
 {
 	GList *node;
-	
+
+	te->force_pref = override_by_pref;
 	node = te->views;
 	while (node)
 	{
-		text_editor_hilite_one (te, GPOINTER_TO_INT (node->data),
-								override_by_pref);
+		text_editor_hilite_one (te, GPOINTER_TO_INT (node->data));
 		node = g_list_next (node);
 	}
 }
@@ -752,6 +793,25 @@ text_editor_get_attribute (TextEditor *te, gint position)
 					break;
 			}
 			break;
+		case SCLEX_PYTHON:
+			switch (style)
+			{
+				case SCE_P_CHARACTER:
+				case SCE_P_STRING:
+				case SCE_P_TRIPLE:
+				case SCE_P_TRIPLEDOUBLE:
+					attrib = TEXT_EDITOR_ATTRIB_STRING;
+					break;
+				case SCE_P_COMMENTLINE:
+				case SCE_P_COMMENTBLOCK:
+					attrib = TEXT_EDITOR_ATTRIB_COMMENT;
+					break;
+				case SCE_P_WORD:
+				case SCE_P_WORD2:
+					attrib = TEXT_EDITOR_ATTRIB_KEYWORD;
+					break;
+			}
+			break;
 	}
 	return attrib;
 }
@@ -2185,10 +2245,13 @@ itext_editor_set_use_spaces (IAnjutaEditor *editor, gboolean use_spaces, GError
 							SCI_SETUSETABS, !use_spaces, 0);
 }
 
+/* This function does not set automatic indentation but allow or not
+ * the editor to indent the text. Used to disable editor indentation when
+ * another plugin take care of it. */
 static void
 itext_editor_set_auto_indent (IAnjutaEditor *editor, gboolean auto_indent, GError **e)
 {
-  text_editor_command (TEXT_EDITOR(editor), ANE_SETINDENTMAINTAIN, auto_indent, 0);
+	text_editor_command (TEXT_EDITOR(editor), ANE_SETAUTOINDENTATION, auto_indent, 0);
 }
 
 static void
@@ -3113,12 +3176,7 @@ iassist_show_tips (IAnjutaEditorAssist *iassist, GList* tips,
 	gint tips_count;
 	TextEditor *te = TEXT_EDITOR (iassist);
 	TextEditorCell *cell = TEXT_EDITOR_CELL (position);
-	gint column;
-	gint pos;
 	gint calltip_pos;
-	gint line;
-	gint calltip_lines = 0;
-	int i;
 	
 	g_return_if_fail (IS_TEXT_EDITOR (te));
 	g_return_if_fail (tips != NULL);
@@ -3136,23 +3194,11 @@ iassist_show_tips (IAnjutaEditorAssist *iassist, GList* tips,
 		g_string_append (calltip, (gchar*) tip->data);
 		tip = g_list_next (tip);
 	}
-	for (i = 0; calltip->str[i] != '\0'; i++)
-	{
-		if (calltip->str[i] == '\n')
-			calltip_lines++;
-	}
-	/* Calculate real calltip position */
-	pos = text_editor_cell_get_position (cell);
-	column = scintilla_send_message (SCINTILLA (te->scintilla),
-	                                 SCI_GETCOLUMN,
-	                                 pos, 0);
-	line = scintilla_send_message (SCINTILLA (te->scintilla),
-	                               SCI_LINEFROMPOSITION,
-	                               pos, 0);
-	/* Align calltip above function call */
-	line -= (calltip_lines + 3);
-	calltip_pos = scintilla_send_message (SCINTILLA(te->scintilla),
-	                                      SCI_POSITIONFROMLINE, line, 0) + column;
+	
+	/* It is not possible to display the calltip above, as the position is defined
+	 * in characters. We cannot be sure that there is enough characters in
+	 * the line above */
+	calltip_pos = text_editor_cell_get_position (cell);
 	
 	scintilla_send_message (SCINTILLA (te->scintilla),
 							SCI_CALLTIPSHOW,
